#include "../../include/http/identityexportxml.h"

#ifdef XMEM
	#include <xmem.h>
#endif

IdentityExportXML::IdentityExportXML()
{
	Initialize();
}

void IdentityExportXML::AddIdentity(const std::string &name, const std::string &publickey, const std::string &privatekey, const bool singleuse, const bool publishtrustlist, const bool publishboardlist)
{
	m_identities.push_back(identity(name,publickey,privatekey,singleuse,publishtrustlist,publishboardlist));
}

const std::string IdentityExportXML::GetName(const long index)
{
	if(index>=0 && index<GetCount())
	{
		return m_identities[index].m_name;
	}
	else
	{
		return "";
	}
}

const std::string IdentityExportXML::GetPrivateKey(const long index)
{
	if(index>=0 && index<GetCount())
	{
		return m_identities[index].m_privatekey;
	}
	else
	{
		return "";
	}
}

const std::string IdentityExportXML::GetPublicKey(const long index)
{
	if(index>=0 && index<GetCount())
	{
		return m_identities[index].m_publickey;
	}
	else
	{
		return "";
	}
}

const bool IdentityExportXML::GetPublishBoardList(const long index)
{
	if(index>=0 && index<GetCount())
	{
		return m_identities[index].m_publishboardlist;
	}
	else
	{
		return false;
	}
}

const bool IdentityExportXML::GetPublishTrustList(const long index)
{
	if(index>=0 && index<GetCount())
	{
		return m_identities[index].m_publishtrustlist;
	}
	else
	{
		return false;
	}
}

const bool IdentityExportXML::GetSingleUse(const long index)
{
	if(index>=0 && index<GetCount())
	{
		return m_identities[index].m_singleuse;
	}
	else
	{
		return false;
	}
}

std::string IdentityExportXML::GetXML()
{
	TiXmlDocument td;
	TiXmlDeclaration *tdec=new TiXmlDeclaration("1.0","UTF-8","");
	TiXmlElement *tid;
	TiXmlPrinter tp;

	td.LinkEndChild(tdec);
	tid=new TiXmlElement("IdentityExport");
	td.LinkEndChild(tid);

	for(std::vector<identity>::iterator i=m_identities.begin(); i!=m_identities.end(); i++)
	{
		TiXmlElement *tr=new TiXmlElement("Identity");
		tid->LinkEndChild(tr);
		tr->LinkEndChild(XMLCreateCDATAElement("Name",(*i).m_name));
		tr->LinkEndChild(XMLCreateTextElement("PublicKey",(*i).m_publickey));
		tr->LinkEndChild(XMLCreateTextElement("PrivateKey",(*i).m_privatekey));
		tr->LinkEndChild(XMLCreateBooleanElement("SingleUse",(*i).m_singleuse));
		tr->LinkEndChild(XMLCreateBooleanElement("PublishTrustList",(*i).m_publishtrustlist));
		tr->LinkEndChild(XMLCreateBooleanElement("PublishBoardList",(*i).m_publishboardlist));
	}

	td.Accept(&tp);
	return std::string(tp.CStr());
}

void IdentityExportXML::Initialize()
{
	m_identities.clear();
}

const bool IdentityExportXML::ParseXML(const std::string &xml)
{
	TiXmlDocument td;
	td.Parse(xml.c_str());

	if(!td.Error())
	{
		std::string name;
		std::string publickey;
		std::string privatekey;
		bool singleuse=false;
		bool publishtrustlist=false;
		bool publishboardlist=false;
		TiXmlText *txt;
		TiXmlHandle hnd(&td);
		TiXmlNode *node;

		Initialize();

		node=hnd.FirstChild("IdentityExport").FirstChild("Identity").ToElement();
		while(node)
		{
			name="";
			publickey="";
			privatekey="";
			singleuse=false;
			publishtrustlist=false;
			publishboardlist=false;

			TiXmlHandle hnd2(node);
			txt=hnd2.FirstChild("Name").FirstChild().ToText();
			if(txt)
			{
				name=txt->ValueStr();
			}
			txt=hnd2.FirstChild("PublicKey").FirstChild().ToText();
			if(txt)
			{
				publickey=txt->ValueStr();
			}
			txt=hnd2.FirstChild("PrivateKey").FirstChild().ToText();
			if(txt)
			{
				privatekey=txt->ValueStr();
			}

			singleuse=XMLGetBooleanElement(node->ToElement(),"SingleUse");
			publishtrustlist=XMLGetBooleanElement(node->ToElement(),"PublishTrustList");
			publishboardlist=XMLGetBooleanElement(node->ToElement(),"PublishBoardList");

			if(name!="" && publickey!="" && privatekey!="")
			{
				m_identities.push_back(identity(name,publickey,privatekey,singleuse,publishtrustlist,publishboardlist));
			}
			
			node=node->NextSibling("Identity");
		}
		return true;

	}
	else
	{
		return false;
	}
}
